/**
 *    DimensionExtent.java
 *    Copyright (C) 2010 New Zealand Digital Library, http://expeditee.org
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.expeditee.io.flowlayout;

import java.util.List;

import org.expeditee.core.bounds.AxisAlignedBoxBounds;
import org.expeditee.core.bounds.PolygonBounds;
import org.expeditee.items.Item;

public class DimensionExtent 
{
	public int min;
	public int max;
	
	public DimensionExtent(int min, int max)
	{
		this.min = min;
		this.max = max;
	}
	
	public static DimensionExtent calcMinMaxXExtent(List<Item>items)
	{
		int min_x = Integer.MAX_VALUE;
		int max_x = Integer.MIN_VALUE + 1; // +1 makes it safe to do math.abs on
		
		for (Item item : items) {
			AxisAlignedBoxBounds bounds = item.getBoundingBox();
			int xl = bounds.getMinX();
			int xr = xl + bounds.getWidth()-1;
			
			min_x = Math.min(min_x, xl);
			max_x = Math.max(max_x, xr);
			
		}
		
		DimensionExtent extent = new DimensionExtent(min_x,max_x);
		
		return extent;
	}
	
	public static DimensionExtent calcMinMaxYExtent(List<Item>items)
	{
		int min_y = Integer.MAX_VALUE;
		int max_y = Integer.MIN_VALUE + 1; // +1 makes it safe to do math.abs on
		
		for (Item item : items) {
			AxisAlignedBoxBounds bounds = item.getBoundingBox();
			int yt = bounds.getMinY();
			int yb = yt + bounds.getHeight()-1;
			
			min_y = Math.min(min_y, yt);
			max_y = Math.max(max_y, yb);
			
		}
		
		DimensionExtent extent = new DimensionExtent(min_y,max_y);
		
		return extent;
	}

	
	public static DimensionExtent calcMinMaxXExtent(PolygonBounds polygon)
	{

		DimensionExtent extent = new DimensionExtent(polygon.getMinX(), polygon.getMaxX());
		
		return extent;
	}
	
	public static DimensionExtent calcMinMaxYExtent(PolygonBounds polygon)
	{

		DimensionExtent extent = new DimensionExtent(polygon.getMinY(), polygon.getMaxY());
		
		return extent;
	}
	
	public static PolygonBounds boundingBoxPolygon(List<Item> items)
	{
		AxisAlignedBoxBounds box = null;
		
		for (Item i : items) {
			if (box == null) {
				box = i.getBoundingBox();
			} else {
				box.combineWith(i.getBoundingBox());
			}
		}
		
		return new PolygonBounds(box.getBorderLines());
	}
}
